@@ -14,6 +14,7 @@ from toolz.curried import identity
 
 import dask
 import dask.array as da
+from dask.delayed import delayed
 from dask.async import get_sync
 from dask.array.core import *
 from dask.utils import raises, ignoring, tmpfile
@@ -86,16 +87,6 @@ def test_top_supports_broadcasting_rules():
          ('z', 1, 1): (add, ('x', 0, 1), ('y', 1, 0))}
 
 
-def test_concatenate3():
-    x = np.array([1, 2])
-    assert concatenate3([[x, x, x],
-                            [x, x, x]]).shape == (2, 6)
-
-    x = np.array([[1, 2]])
-    assert concatenate3([[x, x, x],
-                            [x, x, x]]).shape == (2, 6)
-
-
 def test_concatenate3_on_scalars():
     assert_eq(concatenate3([1, 2]), np.array([1, 2]))
 
@@ -216,27 +207,27 @@ def test_stack():
 
     assert s.shape == (3, 4, 6)
     assert s.chunks == ((1, 1, 1), (2, 2), (3, 3))
-    assert s.dask[(s.name, 0, 1, 0)] == (getarray, ('A', 1, 0),
+    assert s.dask[(s.name, 0, 1, 0)] == (getitem, ('A', 1, 0),
                                           (None, colon, colon))
-    assert s.dask[(s.name, 2, 1, 0)] == (getarray, ('C', 1, 0),
+    assert s.dask[(s.name, 2, 1, 0)] == (getitem, ('C', 1, 0),
                                           (None, colon, colon))
     assert same_keys(s, stack([a, b, c], axis=0))
 
     s2 = stack([a, b, c], axis=1)
     assert s2.shape == (4, 3, 6)
     assert s2.chunks == ((2, 2), (1, 1, 1), (3, 3))
-    assert s2.dask[(s2.name, 0, 1, 0)] == (getarray, ('B', 0, 0),
+    assert s2.dask[(s2.name, 0, 1, 0)] == (getitem, ('B', 0, 0),
                                             (colon, None, colon))
-    assert s2.dask[(s2.name, 1, 1, 0)] == (getarray, ('B', 1, 0),
+    assert s2.dask[(s2.name, 1, 1, 0)] == (getitem, ('B', 1, 0),
                                             (colon, None, colon))
     assert same_keys(s2, stack([a, b, c], axis=1))
 
     s2 = stack([a, b, c], axis=2)
     assert s2.shape == (4, 6, 3)
     assert s2.chunks == ((2, 2), (3, 3), (1, 1, 1))
-    assert s2.dask[(s2.name, 0, 1, 0)] == (getarray, ('A', 0, 1),
+    assert s2.dask[(s2.name, 0, 1, 0)] == (getitem, ('A', 0, 1),
                                             (colon, colon, None))
-    assert s2.dask[(s2.name, 1, 1, 2)] == (getarray, ('C', 1, 1),
+    assert s2.dask[(s2.name, 1, 1, 2)] == (getitem, ('C', 1, 1),
                                             (colon, colon, None))
     assert same_keys(s2, stack([a, b, c], axis=2))
 
@@ -379,9 +370,8 @@ def test_binops():
                                      for i in range(3)))
 
     result = elemwise(pow, a, 2, name='c')
-    assert result.dask[('c', 0)][1] == ('a', 0)
-    f = result.dask[('c', 0)][0]
-    assert f(10) == 100
+    assert "'a', 0" in str(result.dask[('c', 0)])
+    assert "2" in str(result.dask[('c', 0)])
 
 
 def test_isnull():
@@ -422,16 +412,7 @@ def test_elemwise_on_scalars():
 
 
 def test_partial_by_order():
-    f = partial_by_order(add, [(1, 20)])
-    assert f(5) == 25
-    assert f.__name__ == 'add(20)'
-
-    f = partial_by_order(lambda x, y, z: x + y + z, [(1, 10), (2, 15)])
-    assert f(3) == 28
-    assert f.__name__ == '<lambda>(...)'
-
-    assert raises(ValueError, lambda: partial_by_order(add, 1))
-    assert raises(ValueError, lambda: partial_by_order(add, [1]))
+    assert partial_by_order(5, function=add, other=[(1, 20)]) == 25
 
 
 def test_elemwise_with_ndarrays():
@@ -504,12 +485,15 @@ def test_field_access():
 
 def test_tensordot():
     x = np.arange(400).reshape((20, 20))
-    a = from_array(x, chunks=(5, 5))
+    a = from_array(x, chunks=(5, 4))
     y = np.arange(200).reshape((20, 10))
-    b = from_array(y, chunks=(5, 5))
+    b = from_array(y, chunks=(4, 5))
+
+    for axes in [1, (1, 0)]:
+        assert_eq(tensordot(a, b, axes=axes), np.tensordot(x, y, axes=axes))
+        assert_eq(tensordot(x, b, axes=axes), np.tensordot(x, y, axes=axes))
+        assert_eq(tensordot(a, y, axes=axes), np.tensordot(x, y, axes=axes))
 
-    assert_eq(tensordot(a, b, axes=1), np.tensordot(x, y, axes=1))
-    assert_eq(tensordot(a, b, axes=(1, 0)), np.tensordot(x, y, axes=(1, 0)))
     assert same_keys(tensordot(a, b, axes=(1, 0)), tensordot(a, b, axes=(1, 0)))
     assert not same_keys(tensordot(a, b, axes=0), tensordot(a, b, axes=1))
 
@@ -1431,7 +1415,7 @@ def test_histogram_extra_args_and_shapes():
            da.histogram(v, bins=bins, weights=w, density=True)[0])
 
 
-def test_concatenate3():
+def test_concatenate3_2():
     x = np.array([1, 2])
     assert_eq(concatenate3([x, x, x]),
        np.array([1, 2, 1, 2, 1, 2]))
@@ -1859,11 +1843,11 @@ def test_timedelta_op():
     assert a.compute() == x / y
 
 
-def test_to_imperative():
+def test_to_delayed():
     x = da.random.random((4, 4), chunks=(2, 2))
     y = x + 10
 
-    [[a, b], [c, d]] = y.to_imperative()
+    [[a, b], [c, d]] = y.to_delayed()
     assert_eq(a.compute(), y[:2, :2])
 
 
@@ -1996,3 +1980,33 @@ def test_atop_names():
     y = atop(add, 'i', x, 'i')
     assert y.name.startswith('add')
 
+
+def test_atop_kwargs():
+    def f(a, b=0):
+        return a + b
+
+    x = da.ones(5, chunks=(2,))
+    y = atop(f, 'i', x, 'i', b=10, dtype=x.dtype)
+    assert_eq(y, np.ones(5) + 10)
+
+
+def test_from_delayed():
+    v = delayed(np.ones)((5, 3))
+    x = from_delayed(v, shape=(5, 3), dtype=np.ones(0).dtype)
+    assert isinstance(x, Array)
+    assert_eq(x, np.ones((5, 3)))
+
+
+def test_A_property():
+    x = da.ones(5, chunks=(2,))
+    assert x.A is x
+
+
+def test_copy():
+    x = da.ones(5, chunks=(2,))
+    assert x.copy() is x
+
+
+def test_npartitions():
+    assert da.ones(5, chunks=(2,)).npartitions == 3
+    assert da.ones((5, 5), chunks=(2, 3)).npartitions == 6
