@@ -1,35 +1,33 @@
 from __future__ import absolute_import, division, print_function
 
-import io
-import itertools
-import math
-import bz2
-import os
-import uuid
-from fnmatch import fnmatchcase
-from glob import glob
 from collections import Iterable, Iterator, defaultdict
 from functools import wraps, partial
+import itertools
+import math
+import os
+import uuid
+from warnings import warn
 
 from ..utils import ignoring
 
-from toolz import (merge, frequencies, merge_with, take, reduce,
-                   join, reduceby, valmap, count, map, partition_all, filter,
-                   remove, pluck, groupby, topk, compose, curry)
+from toolz import (merge, take, reduce, valmap, map, partition_all, filter,
+        remove, compose, curry)
 from toolz.compatibility import iteritems, zip
 import toolz
-with ignoring(ImportError):
+try:
     from cytoolz import (frequencies, merge_with, join, reduceby,
                          count, pluck, groupby, topk)
+except:
+    from toolz import (frequencies, merge_with, join, reduceby,
+                         count, pluck, groupby, topk)
 
 from ..base import Base, normalize_token, tokenize
-from ..compatibility import (apply, BytesIO, unicode, urlopen, urlparse,
-                             GzipFile)
+from ..compatibility import apply, unicode, urlopen
 from ..core import list2, quote, istask, get_dependencies, reverse_dict
 from ..multiprocessing import get as mpget
 from ..optimize import fuse, cull, inline
-from ..utils import (file_size, infer_compression, open, system_encoding,
-                     takes_multiple_arguments, textblock, funcname)
+from ..utils import (infer_compression, open, system_encoding,
+                     takes_multiple_arguments, funcname)
 
 no_default = '__no__default__'
 
@@ -66,7 +64,7 @@ def lazify(dsk):
     return valmap(lazify_task, dsk)
 
 
-def inline_singleton_lists(dsk):
+def inline_singleton_lists(dsk, dependencies=None):
     """ Inline lists that are only used once
 
     >>> d = {'b': (list, 'a'),
@@ -76,8 +74,8 @@ def inline_singleton_lists(dsk):
 
     Pairs nicely with lazify afterwards
     """
-
-    dependencies = dict((k, get_dependencies(dsk, k)) for k in dsk)
+    if dependencies is None:
+        dependencies = dict((k, get_dependencies(dsk, k)) for k in dsk)
     dependents = reverse_dict(dependencies)
 
     keys = [k for k, v in dsk.items()
@@ -87,9 +85,9 @@ def inline_singleton_lists(dsk):
 
 def optimize(dsk, keys, **kwargs):
     """ Optimize a dask from a dask.bag """
-    dsk2 = cull(dsk, keys)
-    dsk3 = fuse(dsk2, keys)
-    dsk4 = inline_singleton_lists(dsk3)
+    dsk2, dependencies = cull(dsk, keys)
+    dsk3, dependencies = fuse(dsk2, keys, dependencies)
+    dsk4 = inline_singleton_lists(dsk3, dependencies)
     dsk5 = lazify(dsk4)
     return dsk5
 
@@ -217,12 +215,17 @@ class Item(Base):
 
     @staticmethod
     def from_imperative(value):
-        """ Create bag item from an imperative value
+        warn("Deprecation warning: moved to from_delayed")
+        return from_delayed(value)
+
+    @staticmethod
+    def from_delayed(value):
+        """ Create bag item from a dask.delayed value
 
         Parameters
         ----------
         value: a Value
-            A single dask.imperative.Value object, such as come from dask.do
+            A single dask.delayed.Value object, such as come from dask.do
 
         Returns
         -------
@@ -230,9 +233,9 @@ class Item(Base):
 
         Examples
         --------
-        >>> b = db.Item.from_imperative(x)  # doctest: +SKIP
+        >>> b = db.Item.from_delayed(x)  # doctest: +SKIP
         """
-        from dask.imperative import Value
+        from dask.delayed import Value
         assert isinstance(value, Value)
         return Item(value.dask, value.key)
 
@@ -252,11 +255,15 @@ class Item(Base):
     __int__ = __float__ = __complex__ = __bool__ = Base.compute
 
     def to_imperative(self):
+        warn("Deprecation warning: moved to to_delayed")
+        return self.to_delayed()
+
+    def to_delayed(self):
         """ Convert bag item to dask Value
 
         Returns a single value.
         """
-        from dask.imperative import Value
+        from dask.delayed import Value
         return Value(self.key, [self.dask])
 
 
@@ -265,7 +272,6 @@ class Bag(Base):
 
     Examples
     --------
-
     Create Bag from sequence
 
     >>> import dask.bag as db
@@ -275,7 +281,7 @@ class Bag(Base):
 
     Create Bag from filename or globstring of filenames
 
-    >>> b = db.from_filenames('/path/to/mydata.*.json.gz').map(json.loads)  # doctest: +SKIP
+    >>> b = db.read_text('/path/to/mydata.*.json.gz').map(json.loads)  # doctest: +SKIP
 
     Create manually (expert use)
 
@@ -300,6 +306,12 @@ class Bag(Base):
         self.npartitions = npartitions
         self.str = StringAccessor(self)
 
+    def __str__(self):
+        name = self.name if len(self.name) < 10 else self.name[:7] + '...'
+        return 'dask.bag<%s, npartitions=%d>' % (name, self.npartitions)
+
+    __repr__ = __str__
+
     def map(self, func, **kwargs):
         """ Map a function across all elements in collection
 
@@ -453,16 +465,6 @@ class Bag(Base):
                        for i in range(self.npartitions))
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
-    @classmethod
-    def from_sequence(cls, *args, **kwargs):
-        raise AttributeError("db.Bag.from_sequence is deprecated.\n"
-                             "Use db.from_sequence instead.")
-
-    @classmethod
-    def from_filenames(cls, *args, **kwargs):
-        raise AttributeError("db.Bag.from_filenames is deprecated.\n"
-                             "Use db.from_filenames instead.")
-
     @wraps(to_textfiles)
     def to_textfiles(self, path, name_function=str, compression='infer',
                      encoding=system_encoding, compute=True):
@@ -946,11 +948,15 @@ class Bag(Base):
                             name, columns, divisions)
 
     def to_imperative(self):
+        warn("Deprecation warning: moved to to_delayed")
+        return self.to_delayed()
+
+    def to_delayed(self):
         """ Convert bag to dask Values
 
         Returns list of values, one value per partition.
         """
-        from dask.imperative import Value
+        from dask.delayed import Value
         return [Value(k, [self.dask]) for k in self._keys()]
 
     def repartition(self, npartitions):
@@ -998,67 +1004,11 @@ def collect(grouper, group, p, barrier_token):
 
 def from_filenames(filenames, chunkbytes=None, compression='infer',
                    encoding=system_encoding, linesep=os.linesep):
-    """ Create dask by loading in lines from many files
-
-    Provide list of filenames
-
-    >>> b = from_filenames(['myfile.1.txt', 'myfile.2.txt'])  # doctest: +SKIP
-
-    Or a globstring
-
-    >>> b = from_filenames('myfiles.*.txt')  # doctest: +SKIP
-
-    Parallelize a large files by providing the number of uncompressed bytes to
-    load into each partition.
-
-    >>> b = from_filenames('largefile.txt', chunkbytes=1e7)  # doctest: +SKIP
-
-    See Also
-    --------
-    from_sequence: A more generic bag creation function
-    """
-    if isinstance(filenames, str):
-        filenames = sorted(glob(filenames))
-
-    if not filenames:
-        raise ValueError("No filenames found")
-
-    full_filenames = [os.path.abspath(f) for f in filenames]
-
-    name = 'from-filenames-' + uuid.uuid4().hex
-
-    # Make sure `linesep` is not a byte string because `io.TextIOWrapper` in
-    # python versions other than 2.7 dislike byte strings for the `newline`
-    # argument.
-    linesep = str(linesep)
-
-    def get_compression(path, compression=compression):
-        if compression == 'infer':
-            compression = infer_compression(path)
-        return compression
-
-    if chunkbytes:
-        chunkbytes = int(chunkbytes)
-        taskss = [_chunk_read_file(fn, chunkbytes, get_compression(fn),
-                                   encoding, linesep)
-                  for fn in full_filenames]
-        d = dict(((name, i), task)
-                 for i, task in enumerate(toolz.concat(taskss)))
-    else:
-        d = dict(((name, i), (list,
-                              (io.TextIOWrapper,
-                               (io.BufferedReader,
-                                (open, fn, 'rb', get_compression(fn))),
-                               encoding, None, linesep)))
-                 for i, fn in enumerate(full_filenames))
-
-    return Bag(d, name, len(d))
-
-
-def _chunk_read_file(filename, chunkbytes, compression, encoding, linesep):
-    return [(list, (textblock, filename, i, i + chunkbytes, compression,
-                    encoding, linesep))
-            for i in range(0, file_size(filename, compression), chunkbytes)]
+    """ Deprecated.  See read_text """
+    warn("db.from_filenames is deprecated in favor of db.read_text")
+    from .text import read_text
+    return read_text(filenames, blocksize=chunkbytes, compression=compression,
+            encoding=encoding, linedelimiter=linesep)
 
 
 def write(data, filename, compression, encoding):
@@ -1089,106 +1039,6 @@ def write(data, filename, compression, encoding):
         f.close()
 
 
-def _get_s3_bucket(bucket_name, aws_access_key, aws_secret_key, connection,
-                   anon):
-    """Connect to s3 and return a bucket"""
-    import boto
-    if anon is True:
-        connection = boto.connect_s3(anon=anon)
-    elif connection is None:
-        connection = boto.connect_s3(aws_access_key, aws_secret_key)
-    return connection.get_bucket(bucket_name)
-
-
-# we need an unmemoized function to call in the main thread. And memoized
-# functions for the dask.
-_memoized_get_bucket = toolz.memoize(_get_s3_bucket)
-
-
-def _get_key(bucket_name, conn_args, key_name):
-    bucket = _memoized_get_bucket(bucket_name, *conn_args)
-    key = bucket.get_key(key_name)
-    ext = key_name.split('.')[-1]
-    return stream_decompress(ext, key.read())
-
-
-def _parse_s3_URI(bucket_name, paths):
-    from ..compatibility import quote, unquote
-    assert bucket_name.startswith('s3://')
-    o = urlparse('s3://' + quote(bucket_name[len('s3://'):]))
-    # if path is specified
-    if (paths == '*') and (o.path != '' and o.path != '/'):
-        paths = unquote(o.path[1:])
-    bucket_name = unquote(o.hostname)
-    return bucket_name, paths
-
-
-def from_s3(bucket_name, paths='*', aws_access_key=None, aws_secret_key=None,
-            connection=None, anon=False):
-    """ Create a Bag by loading textfiles from s3
-
-    Each line will be treated as one element and each file in S3 as one
-    partition.
-
-    You may specify a full s3 bucket
-
-    >>> b = from_s3('s3://bucket-name')  # doctest: +SKIP
-
-    Or select files, lists of files, or globstrings of files within that bucket
-
-    >>> b = from_s3('s3://bucket-name', 'myfile.json')  # doctest: +SKIP
-    >>> b = from_s3('s3://bucket-name', ['alice.json', 'bob.json'])  # doctest: +SKIP
-    >>> b = from_s3('s3://bucket-name', '*.json')  # doctest: +SKIP
-    """
-    conn_args = (aws_access_key, aws_secret_key, connection, anon)
-
-    bucket_name, paths = normalize_s3_names(bucket_name, paths, conn_args)
-
-    get_key = partial(_get_key, bucket_name, conn_args)
-
-    name = 'from_s3-' + uuid.uuid4().hex
-    dsk = dict(((name, i), (list, (get_key, k))) for i, k in enumerate(paths))
-    return Bag(dsk, name, len(paths))
-
-
-def normalize_s3_names(bucket_name, paths, conn_args):
-    """ Normalize bucket name and paths """
-    if bucket_name.startswith('s3://'):
-        bucket_name, paths = _parse_s3_URI(bucket_name, paths)
-
-    if isinstance(paths, str):
-        if ('*' not in paths) and ('?' not in paths):
-            return bucket_name, [paths]
-        else:
-            bucket = _get_s3_bucket(bucket_name, *conn_args)
-            keys = bucket.list()  # handle globs
-
-            matches = [k.name for k in keys if fnmatchcase(k.name, paths)]
-            return bucket_name, matches
-    else:
-        return bucket_name, paths
-
-
-def stream_decompress(fmt, data):
-    """ Decompress a block of compressed bytes into a stream of strings """
-    if fmt == 'gz':
-        return GzipFile(fileobj=BytesIO(data))
-    if fmt == 'bz2':
-        return bz2_stream(data)
-    else:
-        return map(bytes.decode, BytesIO(data))
-
-
-def bz2_stream(compressed, chunksize=100000):
-    """ Stream lines from a chunk of compressed bz2 data """
-    decompressor = bz2.BZ2Decompressor()
-    for i in range(0, len(compressed), chunksize):
-        chunk = compressed[i: i+chunksize]
-        decompressed = decompressor.decompress(chunk).decode()
-        for line in decompressed.split('\n'):
-            yield line + '\n'
-
-
 def from_sequence(seq, partition_size=None, npartitions=None):
     """ Create dask from Python sequence
 
@@ -1198,7 +1048,6 @@ def from_sequence(seq, partition_size=None, npartitions=None):
 
     Parameters
     ----------
-
     seq: Iterable
         A sequence of elements to put into the dask
     partition_size: int (optional)
@@ -1211,12 +1060,11 @@ def from_sequence(seq, partition_size=None, npartitions=None):
 
     Examples
     --------
-
     >>> b = from_sequence(['Alice', 'Bob', 'Chuck'], partition_size=2)
 
     See Also
     --------
-    from_filenames: Specialized bag creation function for textfiles
+    read_text: Create bag from textfiles
     """
     seq = list(seq)
     if npartitions and not partition_size:
@@ -1409,12 +1257,17 @@ def reify(seq):
 
 
 def from_imperative(values):
-    """ Create bag from many imperative objects
+    warn("Deprecation warning: moved to from_delayed")
+    return from_delayed(values)
+
+
+def from_delayed(values):
+    """ Create bag from many dask.delayed objects
 
     Parameters
     ----------
     values: list of Values
-        An iterable of dask.imperative.Value objects, such as come from dask.do
+        An iterable of dask.delayed.Value objects, such as come from dask.do
         These comprise the individual partitions of the resulting bag
 
     Returns
@@ -1423,14 +1276,14 @@ def from_imperative(values):
 
     Examples
     --------
-    >>> b = from_imperative([x, y, z])  # doctest: +SKIP
+    >>> b = from_delayed([x, y, z])  # doctest: +SKIP
     """
-    from dask.imperative import Value
+    from dask.delayed import Value
     if isinstance(values, Value):
         values = [values]
     dsk = merge(v.dask for v in values)
 
-    name = 'bag-from-imperative-' + tokenize(*values)
+    name = 'bag-from-delayed-' + tokenize(*values)
     names = [(name, i) for i in range(len(values))]
     values = [v.key for v in values]
     dsk2 = dict(zip(names, values))
