@@ -1,8 +1,8 @@
 from .core import (DataFrame, Series, Index, _Frame, map_partitions,
                    repartition)
-from .io import (read_csv, from_array, from_bcolz, from_array, from_bcolz,
+from .io import (from_array, from_bcolz, from_array, from_bcolz,
                  from_pandas, from_dask_array, from_castra, read_hdf,
-                 from_imperative)
+                 from_imperative, from_delayed)
 from .optimize import optimize
 from .multi import merge, concat, melt
 from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
@@ -10,4 +10,5 @@ from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
                       rolling_skew, rolling_kurt, rolling_quantile, rolling_apply,
                       rolling_window)
 from ..base import compute
+from .csv import read_csv
 from . import demo
