@@ -1,263 +1,29 @@
 from __future__ import absolute_import, division, print_function
 
-import codecs
 from fnmatch import fnmatch
 from functools import wraps
 from glob import glob
 from math import ceil
 from operator import getitem
 import os
-import re
 from threading import Lock
 import uuid
+from warnings import warn
 
 import pandas as pd
 import numpy as np
-from toolz import merge, assoc, dissoc
+from toolz import merge
 
-from ..compatibility import StringIO, unicode, range, apply
-from ..utils import (textblock, file_size, get_bom, system_encoding,
-                     infer_compression)
 from ..base import tokenize
+from ..compatibility import unicode, apply
 from .. import array as da
 from ..async import get_sync
 
-from . import core
 from .core import _Frame, DataFrame, Series
 from .shuffle import set_partition
 
 
 lock = Lock()
-csv_defaults = {'compression': 'infer'}
-
-
-def _read_csv(fn, i, chunkbytes, compression, kwargs, bom):
-    kwargs = kwargs.copy()
-    linesep = kwargs.get('lineterminator', os.linesep)
-    encoding = kwargs.pop('encoding', system_encoding)
-
-    start = i * chunkbytes
-    end = start + chunkbytes
-
-    if encoding == 'utf-16':
-        bom_encoding = {codecs.BOM_UTF16_BE: 'utf-16-be',
-                        codecs.BOM_UTF16_LE: 'utf-16-le'}
-        if not bom:
-            bom = codecs.BOM_UTF16
-        if i > 0:
-            encoding = bom_encoding[bom]
-
-    block = StringIO(u''.join(textblock(fn, start, end, compression, encoding,
-                                        linesep)))
-    try:
-        return pd.read_csv(block, **kwargs)
-    except ValueError as e:
-        msg = """
-    Dask dataframe inspected the first 1,000 rows of your csv file to guess the
-    data types of your columns.  These first 1,000 rows led us to an incorrect
-    guess.
-
-    For example a column may have had integers in the first 1000
-    rows followed by a float or missing value in the 1,001-st row.
-
-    You will need to specify some dtype information explicitly using the
-    ``dtype=`` keyword argument for the right column names and dtypes.
-
-        df = dd.read_csv(..., dtype={'my-column': float})
-
-    Pandas has given us the following error when trying to parse the file:
-
-      "%s"
-        """ % e.args[0]
-        match = re.match('cannot safely convert passed user dtype of (?P<old_dtype>\S+) for (?P<new_dtype>\S+) dtyped data in column (?P<column_number>\d+)', e.args[0])
-        if match:
-            d = match.groupdict()
-            d['column'] = kwargs['names'][int(d['column_number'])]
-            msg += """
-    From this we think that you should probably add the following column/dtype
-    pair to your dtype= dictionary
-
-    '%(column)s': '%(new_dtype)s'
-        """ % d
-
-        # TODO: add more regexes and msg logic here for other pandas errors
-        #       as apporpriate
-
-        raise ValueError(msg)
-    finally:
-        block.close()
-
-
-def _clean_kwargs(kwargs):
-    """ Do some sanity checks on kwargs
-
-    >>> _clean_kwargs({'parse_dates': ['a', 'b'], 'usecols': ['b', 'c']})
-    {'parse_dates': ['b'], 'usecols': ['b', 'c']}
-
-    >>> _clean_kwargs({'names': ['a', 'b'], 'usecols': [1]})
-    {'parse_dates': [], 'names': ['a', 'b'], 'usecols': ['b']}
-    """
-    kwargs = kwargs.copy()
-
-    if kwargs.get('usecols') and 'names' in kwargs:
-        kwargs['usecols'] = [kwargs['names'][c]
-                              if isinstance(c, int) and c not in kwargs['names']
-                              else c
-                              for c in kwargs['usecols']]
-
-    kwargs['parse_dates'] = [col for col in kwargs.get('parse_dates', ())
-            if kwargs.get('usecols') is None
-            or isinstance(col, (tuple, list)) and all(c in kwargs['usecols']
-                                                      for c in col)
-            or col in kwargs['usecols']]
-
-    return kwargs
-
-
-def _fill_kwargs(fn, **kwargs):
-    """ Read a csv file and fill up kwargs
-
-    This normalizes kwargs against a sample file.  It does the following:
-
-    1.  If given a globstring, just use one file
-    2.  Get names from csv file if not given
-    3.  Identify the presence of a header
-    4.  Identify dtypes
-    5.  Establish column names
-    6.  Switch around dtypes and column names if parse_dates is active
-
-    Normally ``pd.read_csv`` does this for us.  However for ``dd.read_csv`` we
-    need to be consistent across multiple files and don't want to do these
-    heuristics each time so we use the pandas solution once, record the
-    results, and then send back a fully explicit kwargs dict to send to future
-    calls to ``pd.read_csv``.
-
-    Returns
-    -------
-
-    head: pd.DataFrame
-        data read by pd.read_csv
-    kwargs: dict
-        keyword arguments to give to pd.read_csv
-    """
-
-    if 'index_col' in kwargs:
-        msg = """
-        The index column cannot be set at dataframe creation time. Instead use
-        the `set_index` method on the dataframe after it is created.
-        """
-        raise ValueError(msg)
-
-    kwargs = merge(csv_defaults, kwargs)
-    sample_nrows = kwargs.pop('sample_nrows', 1000)
-    essentials = ['columns', 'names', 'header', 'parse_dates', 'dtype']
-    if set(essentials).issubset(kwargs):
-        return kwargs
-
-    # Let pandas infer on the first 100 rows
-    if '*' in fn:
-        filenames = sorted(glob(fn))
-        if not filenames:
-            raise ValueError("No files found matching name %s" % fn)
-        fn = filenames[0]
-
-    if kwargs['compression'] == 'infer':
-        kwargs['compression'] = infer_compression(fn)
-
-    if 'names' not in kwargs:
-        kwargs['names'] = _csv_names(fn, **kwargs)
-        if 'header' not in kwargs:
-            kwargs['header'] = 0
-    else:
-        if 'header' not in kwargs:
-            kwargs['header'] = None
-
-    kwargs = _clean_kwargs(kwargs)
-    try:
-        head = pd.read_csv(fn, **assoc(kwargs, 'nrows', sample_nrows))
-    except StopIteration:
-        head = pd.read_csv(fn, **kwargs)
-
-    if 'parse_dates' not in kwargs:
-        kwargs['parse_dates'] = [col for col in head.dtypes.index
-                           if np.issubdtype(head.dtypes[col], np.datetime64)]
-
-    if 'sep' not in kwargs:
-        new_dtype = dict(head.dtypes)
-        dtype = kwargs.get('dtype', dict())
-        for k, v in dict(head.dtypes).items():
-            if k not in dtype:
-                dtype[k] = v
-
-        if kwargs.get('parse_dates'):
-            for col in kwargs['parse_dates']:
-                if isinstance(col, (tuple, list)):
-                    del dtype['_'.join(col)]
-                else:
-                    del dtype[col]
-
-        kwargs['dtype'] = dtype
-
-    head.columns = head.columns.map(lambda s: s.strip() if isinstance(s, str) else s)
-    return head, kwargs
-
-
-@wraps(pd.read_csv)
-def read_csv(fn, **kwargs):
-    if 'nrows' in kwargs:  # Just create single partition
-        df = read_csv(fn, **dissoc(kwargs, 'nrows'))
-        return df.head(kwargs['nrows'], compute=False)
-
-    chunkbytes = kwargs.pop('chunkbytes', 2**25)  # 50 MB
-    index = kwargs.pop('index', None)
-    kwargs = kwargs.copy()
-
-    head, kwargs = _fill_kwargs(fn, **kwargs)
-
-    # Handle glob strings
-    if '*' in fn:
-        from .multi import concat
-        return concat([read_csv(f, **kwargs) for f in sorted(glob(fn))])
-
-    token = tokenize(os.path.getmtime(fn), kwargs)
-    name = 'read-csv-%s-%s' % (fn, token)
-    bom = get_bom(fn, kwargs.get('compression', None))
-
-    # Chunk sizes and numbers
-    total_bytes = file_size(fn, kwargs['compression'])
-    nchunks = int(ceil(total_bytes / chunkbytes))
-    divisions = [None] * (nchunks + 1)
-
-    first_kwargs = merge(kwargs, dict(compression=None))
-    rest_kwargs = merge(kwargs, dict(header=None, compression=None))
-
-    # Create dask graph
-    dsk = dict(((name, i), (_read_csv, fn, i, chunkbytes,
-                            kwargs['compression'], rest_kwargs, bom))
-               for i in range(1, nchunks))
-
-    dsk[(name, 0)] = (_read_csv, fn, 0, chunkbytes, kwargs['compression'],
-                      first_kwargs, b'')
-
-    result = DataFrame(dsk, name, head, divisions)
-
-    if index:
-        result = result.set_index(index)
-
-    return result
-
-
-def _csv_names(fn, encoding='utf-8', compression=None, names=None,
-               parse_dates=None, usecols=None, dtype=None, **kwargs):
-    try:
-        kwargs['nrows'] = 5
-        df = pd.read_csv(fn, encoding=encoding, compression=compression,
-                         names=names, **kwargs)
-    except StopIteration:
-        kwargs['nrows'] = None
-        df = pd.read_csv(fn, encoding=encoding, compression=compression,
-                         names=names, **kwargs)
-    return list(df.columns)
 
 
 def _dummy_from_array(x, columns=None):
@@ -326,7 +92,7 @@ def from_array(x, chunksize=50000, columns=None):
     return _Frame(dsk, name, dummy, divisions)
 
 
-def from_pandas(data, npartitions=None, chunksize=None, sort=True):
+def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
     """Construct a dask object from a pandas object.
 
     If given a ``pandas.Series`` a ``dask.Series`` will be returned. If given a
@@ -342,7 +108,6 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True):
     chunksize : int, optional
         The size of the partitions of the index.
 
-
     Returns
     -------
     dask.DataFrame or dask.Series
@@ -385,7 +150,9 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True):
 
     if ((npartitions is None) == (chunksize is None)):
         raise ValueError('Exactly one of npartitions and chunksize must be specified.')
+
     nrows = len(data)
+
     if chunksize is None:
         chunksize = int(ceil(nrows / npartitions))
     else:
@@ -394,20 +161,21 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True):
     if sort and not data.index.is_monotonic_increasing:
         data = data.sort_index(ascending=True)
     if sort:
-        divisions = tuple(data.index[i]
-                          for i in range(0, nrows, chunksize))
-        divisions = divisions + (data.index[-1],)
+        divisions, locations = sorted_division_locations(data.index,
+                                                         chunksize=chunksize)
     else:
         divisions = [None] * (npartitions + 1)
+        locations = list(range(0, nrows, chunksize)) + [len(data)]
 
-    name = 'from_pandas-' + tokenize(data, chunksize)
-    dsk = dict(((name, i), data.iloc[i * chunksize:(i + 1) * chunksize])
-               for i in range(npartitions - 1))
-    dsk[(name, npartitions - 1)] = data.iloc[chunksize*(npartitions - 1):]
+    name = name or ('from_pandas-' + tokenize(data, chunksize))
+    dsk = dict(((name, i), data.iloc[start: stop])
+               for i, (start, stop) in enumerate(zip(locations[:-1],
+                   locations[1:])))
     return _Frame(dsk, name, data, divisions)
 
 
-def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
+def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
+               **kwargs):
     """ Read dask Dataframe from bcolz.ctable
 
     Parameters
@@ -422,12 +190,17 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
         Automatically categorize all string dtypes
     index : string, optional
         Column to make the index
+    lock: bool or Lock
+        Lock to use when reading or False for no lock (not-thread-safe)
 
     See Also
     --------
 
     from_array: more generic function not optimized for bcolz
     """
+    if lock is True:
+        lock = Lock()
+
     import dask.array as da
     import bcolz
 
@@ -458,10 +231,10 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
     new_name = 'from_bcolz-' + token
 
     dsk = dict(((new_name, i),
-                (locked_df_from_ctable,
+                (dataframe_from_ctable,
                  x,
                  (slice(i * chunksize, (i + 1) * chunksize),),
-                 columns, categories))
+                 columns, categories, lock))
                for i in range(0, int(ceil(len(x) / chunksize))))
 
     result = DataFrame(dsk, new_name, columns, divisions)
@@ -476,7 +249,7 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
         return result
 
 
-def dataframe_from_ctable(x, slc, columns=None, categories=None):
+def dataframe_from_ctable(x, slc, columns=None, categories=None, lock=lock):
     """ Get DataFrame from bcolz.ctable
 
     Parameters
@@ -519,29 +292,30 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None):
         stop = slc[0].stop if slc[0].stop < len(x) else len(x)
     idx = pd.Index(range(start, stop))
 
-    if isinstance(x, bcolz.ctable):
-        chunks = [x[name][slc] for name in columns]
-        if categories is not None:
-            chunks = [pd.Categorical.from_codes(np.searchsorted(categories[name],
-                                                                chunk),
-                                                categories[name], True)
-                       if name in categories else chunk
-                       for name, chunk in zip(columns, chunks)]
-        return pd.DataFrame(dict(zip(columns, chunks)), columns=columns,
-                            index=idx)
+    if lock:
+        lock.acquire()
+    try:
+        if isinstance(x, bcolz.ctable):
+            chunks = [x[name][slc] for name in columns]
+            if categories is not None:
+                chunks = [pd.Categorical.from_codes(
+                                    np.searchsorted(categories[name], chunk),
+                                    categories[name], True)
+                           if name in categories else chunk
+                           for name, chunk in zip(columns, chunks)]
+            result = pd.DataFrame(dict(zip(columns, chunks)), columns=columns,
+                                index=idx)
 
-    elif isinstance(x, bcolz.carray):
-        chunk = x[slc]
-        if categories is not None and columns and columns in categories:
-            chunk = pd.Categorical.from_codes(
-                        np.searchsorted(categories[columns], chunk),
-                        categories[columns], True)
-        return pd.Series(chunk, name=columns, index=idx)
-
-
-def locked_df_from_ctable(*args, **kwargs):
-    with lock:
-        result = dataframe_from_ctable(*args, **kwargs)
+        elif isinstance(x, bcolz.carray):
+            chunk = x[slc]
+            if categories is not None and columns and columns in categories:
+                chunk = pd.Categorical.from_codes(
+                            np.searchsorted(categories[columns], chunk),
+                            categories[columns], True)
+            result = pd.Series(chunk, name=columns, index=idx)
+    finally:
+        if lock:
+            lock.release()
     return result
 
 
@@ -859,26 +633,32 @@ def to_bag(df, index=False):
     return Bag(dsk, name, df.npartitions)
 
 
-def from_imperative(dfs, metadata=None, divisions=None, columns=None):
-    """ Create DataFrame from many imperative objects
+def from_imperative(*args, **kwargs):
+    warn("Deprecation warning: moved to from_delayed")
+    return from_delayed(*args, **kwargs)
+
+
+def from_delayed(dfs, metadata=None, divisions=None, columns=None,
+                      prefix='from-delayed'):
+    """ Create DataFrame from many dask.delayed objects
 
     Parameters
     ----------
     dfs: list of Values
-        An iterable of dask.imperative.Value objects, such as come from dask.do
+        An iterable of dask.delayed.Value objects, such as come from dask.do
         These comprise the individual partitions of the resulting dataframe
     metadata: list or string of column names or empty dataframe
     divisions: list or None
     """
     if columns is not None:
-        print("Deprecation warning: Use metadata argument, not columns")
+        warn("Deprecation warning: Use metadata argument, not columns")
         metadata = columns
-    from dask.imperative import Value
+    from dask.delayed import Value
     if isinstance(dfs, Value):
         dfs = [dfs]
     dsk = merge(df.dask for df in dfs)
 
-    name = 'from-imperative-' + tokenize(*dfs)
+    name = prefix + '-' + tokenize(*dfs)
     names = [(name, i) for i in range(len(dfs))]
     values = [df.key for df in dfs]
     dsk2 = dict(zip(names, values))
@@ -890,3 +670,51 @@ def from_imperative(dfs, metadata=None, divisions=None, columns=None):
         return Series(merge(dsk, dsk2), name, metadata, divisions)
     else:
         return DataFrame(merge(dsk, dsk2), name, metadata, divisions)
+
+
+def sorted_division_locations(seq, npartitions=None, chunksize=None):
+    """ Find division locations and values in sorted list
+
+    Examples
+    --------
+
+    >>> L = ['A', 'B', 'C', 'D', 'E', 'F']
+    >>> sorted_division_locations(L, chunksize=2)
+    (['A', 'C', 'E', 'F'], [0, 2, 4, 6])
+
+    >>> sorted_division_locations(L, chunksize=3)
+    (['A', 'D', 'F'], [0, 3, 6])
+
+    >>> L = ['A', 'A', 'A', 'A', 'B', 'B', 'B', 'C']
+    >>> sorted_division_locations(L, chunksize=3)
+    (['A', 'B', 'C'], [0, 4, 8])
+
+    >>> sorted_division_locations(L, chunksize=2)
+    (['A', 'B', 'C'], [0, 4, 8])
+
+    >>> sorted_division_locations(['A'], chunksize=2)
+    (['A', 'A'], [0, 1])
+    """
+    if ((npartitions is None) == (chunksize is None)):
+        raise ValueError('Exactly one of npartitions and chunksize must be specified.')
+
+    if npartitions:
+        chunksize = ceil(len(seq) / npartitions)
+
+    positions = [0]
+    values = [seq[0]]
+    for pos in list(range(0, len(seq), chunksize)):
+        if pos <= positions[-1]:
+            continue
+        while pos + 1 < len(seq) and seq[pos - 1] == seq[pos]:
+            pos += 1
+        values.append(seq[pos])
+        if pos == len(seq) - 1:
+            pos += 1
+        positions.append(pos)
+
+    if positions[-1] != len(seq):
+        positions.append(len(seq))
+        values.append(seq[-1])
+
+    return values, positions
