@@ -1,4 +1,3 @@
-from datetime import datetime
 from operator import getitem
 from distutils.version import LooseVersion
 
@@ -14,7 +13,7 @@ import dask.dataframe as dd
 
 from dask.dataframe.core import (repartition_divisions, _loc,
         _coerce_loc_index, aca, reduction, _concat, _Frame)
-from dask.dataframe.utils import eq, assert_dask_graph
+from dask.dataframe.utils import eq
 
 
 dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
@@ -23,14 +22,14 @@ dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
                               index=[5, 6, 8]),
        ('x', 2): pd.DataFrame({'a': [7, 8, 9], 'b': [0, 0, 0]},
                               index=[9, 9, 9])}
-d = dd.DataFrame(dsk, 'x', ['a', 'b'], [0, 4, 9, 9])
+d = dd.DataFrame(dsk, 'x', ['a', 'b'], [0, 5, 9, 9])
 full = d.compute()
 
 
 def test_Dataframe():
     expected = pd.Series([2, 3, 4, 5, 6, 7, 8, 9, 10],
-                        index=[0, 1, 3, 5, 6, 8, 9, 9, 9],
-                        name='a')
+                         index=[0, 1, 3, 5, 6, 8, 9, 9, 9],
+                         name='a')
 
     assert eq(d['a'] + 1, expected)
 
@@ -55,10 +54,10 @@ def test_head_tail():
     assert eq(d['a'].head(2), full['a'].head(2))
     assert eq(d['a'].head(3), full['a'].head(3))
     assert eq(d['a'].head(2), dsk[('x', 0)]['a'].head(2))
-    assert sorted(d.head(2, compute=False).dask) == \
-           sorted(d.head(2, compute=False).dask)
-    assert sorted(d.head(2, compute=False).dask) != \
-           sorted(d.head(3, compute=False).dask)
+    assert (sorted(d.head(2, compute=False).dask) ==
+            sorted(d.head(2, compute=False).dask))
+    assert (sorted(d.head(2, compute=False).dask) !=
+            sorted(d.head(3, compute=False).dask))
 
     assert eq(d.tail(2), full.tail(2))
     assert eq(d.tail(3), full.tail(3))
@@ -66,10 +65,10 @@ def test_head_tail():
     assert eq(d['a'].tail(2), full['a'].tail(2))
     assert eq(d['a'].tail(3), full['a'].tail(3))
     assert eq(d['a'].tail(2), dsk[('x', 2)]['a'].tail(2))
-    assert sorted(d.tail(2, compute=False).dask) == \
-           sorted(d.tail(2, compute=False).dask)
-    assert sorted(d.tail(2, compute=False).dask) != \
-           sorted(d.tail(3, compute=False).dask)
+    assert (sorted(d.tail(2, compute=False).dask) ==
+            sorted(d.tail(2, compute=False).dask))
+    assert (sorted(d.tail(2, compute=False).dask) !=
+            sorted(d.tail(3, compute=False).dask))
 
 
 def test_index_head():
@@ -84,6 +83,17 @@ def test_Series():
     assert repr(d.a).startswith('dd.Series')
 
 
+def test_repr():
+    df = pd.DataFrame({'x': list(range(100))})
+    ddf = dd.from_pandas(df, 3)
+
+    for x in [ddf, ddf.index, ddf.x]:
+        assert type(x).__name__ in repr(x)
+        assert x._name[:5] in repr(x)
+        assert str(x.npartitions) in repr(x)
+        assert len(repr(x)) < 80
+
+
 def test_Index():
     for case in [pd.DataFrame(np.random.randn(10, 5), index=list('abcdefghij')),
                  pd.DataFrame(np.random.randn(10, 5),
@@ -105,7 +115,7 @@ def test_column_names():
     tm.assert_index_equal(d[['b', 'a']].columns, pd.Index(['b', 'a']))
     assert d['a'].name == 'a'
     assert (d['a'] + 1).name == 'a'
-    assert (d['a'] + d['b']).name == None
+    assert (d['a'] + d['b']).name is None
 
 
 def test_index_names():
@@ -142,6 +152,7 @@ def test_set_index():
     assert d4.index.name == 'b'
     assert eq(d4, full.set_index('b'))
 
+
 @pytest.mark.parametrize('drop', [True, False])
 def test_set_index_drop(drop):
 
@@ -176,7 +187,7 @@ def test_set_index_drop(drop):
 
 def test_set_index_raises_error_on_bad_input():
     df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
-                        'b': [7, 6, 5, 4, 3, 2, 1]})
+                       'b': [7, 6, 5, 4, 3, 2, 1]})
     ddf = dd.from_pandas(df, 2)
 
     msg = r"Dask dataframe does not yet support multi-indexes"
@@ -249,6 +260,7 @@ def test_cumulative():
     assert eq(ddf.a.cummin(), pdf.a.cummin())
     assert eq(ddf.a.cummax(), pdf.a.cummax())
 
+
 def test_dropna():
     df = pd.DataFrame({'x': [np.nan, 2,      3, 4, np.nan,      6],
                        'y': [1,      2, np.nan, 4, np.nan, np.nan],
@@ -296,7 +308,7 @@ def test_where_mask():
                          'b': [False, False, True] * 3,
                          'd': [False] * 9,
                          'e': [True] * 9},
-                         index=[5, 6, 7, 8, 9, 10, 11, 12, 13])
+                        index=[5, 6, 7, 8, 9, 10, 11, 12, 13])
     ddf6 = dd.from_pandas(pdf6, 2)
 
     cases = [(ddf1, ddf2, pdf1, pdf2),
@@ -333,6 +345,7 @@ def test_where_mask():
         assert eq(ddf.a.where(ddcond.a, -ddf.a), pdf.a.where(pdcond.a, -pdf.a))
         assert eq(ddf.a.mask(ddcond.a, -ddf.a), pdf.a.mask(pdcond.a, -pdf.a))
 
+
 def test_map_partitions_multi_argument():
     assert eq(dd.map_partitions(lambda a, b: a + b, None, d.a, d.b),
               full.a + full.b)
@@ -376,7 +389,7 @@ def test_map_partitions_column_info():
     assert eq(df.x, b)
 
     b = dd.map_partitions(lambda df: df.x + df.y, None, a)
-    assert b.name == None
+    assert b.name is None
     assert isinstance(b, dd.Series)
 
     b = dd.map_partitions(lambda df: df.x + 1, 'x', a)
@@ -394,7 +407,7 @@ def test_map_partitions_method_names():
 
     b = a.map_partitions(lambda df: df.x + 1, columns=None)
     assert isinstance(b, dd.Series)
-    assert b.name == None
+    assert b.name is None
 
     b = a.map_partitions(lambda df: df.x + 1, columns='x')
     assert isinstance(b, dd.Series)
@@ -405,7 +418,6 @@ def test_map_partitions_keeps_kwargs_in_dict():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [5, 6, 7, 8]})
     a = dd.from_pandas(df, npartitions=2)
 
-
     def f(s, x=1):
         return s + x
 
@@ -434,7 +446,6 @@ def test_drop_duplicates_subset():
     else:
         kwargs = [{'keep': 'first'}, {'keep': 'last'}]
 
-
     for kwarg in kwargs:
         assert eq(df.x.drop_duplicates(**kwarg),
                   ddf.x.drop_duplicates(**kwarg))
@@ -527,6 +538,7 @@ def test_value_counts():
     # https://github.com/pydata/pandas/pull/10419
     assert eq(result, expected, check_names=False)
 
+
 def test_unique():
     pdf = pd.DataFrame({'x': [1, 2, 1, 3, 3, 1, 4, 2, 3, 1],
                         'y': ['a', 'c', 'b', np.nan, 'c',
@@ -546,10 +558,11 @@ def test_len():
     assert len(d) == len(full)
     assert len(d.a) == len(full.a)
 
+
 def test_quantile():
     # series / multiple
     result = d.b.quantile([.3, .7])
-    exp = full.b.quantile([.3, .7]) # result may different
+    exp = full.b.quantile([.3, .7])  # result may different
     assert len(result) == 2
     assert result.divisions == (.3, .7)
     assert eq(result.index, exp.index)
@@ -578,11 +591,12 @@ def test_quantile():
 
     # series / single
     result = d.b.quantile(.5)
-    exp = full.b.quantile(.5) # result may different
+    exp = full.b.quantile(.5)  # result may different
     assert isinstance(result, dd.core.Scalar)
     result = result.compute()
     assert 4 < result < 6
 
+
 def test_empty_quantile():
     result = d.b.quantile([])
     exp = full.b.quantile([])
@@ -594,6 +608,7 @@ def test_empty_quantile():
     assert result.compute().name == 'b'
     assert eq(result, exp, check_names=False)
 
+
 def test_dataframe_quantile():
 
     # column X is for test column order and result division
@@ -601,7 +616,7 @@ def test_dataframe_quantile():
                        'X': np.arange(20, 40),
                        'B': np.arange(10, 30),
                        'C': ['a', 'b', 'c', 'd'] * 5},
-                       columns=['A', 'X', 'B', 'C'])
+                      columns=['A', 'X', 'B', 'C'])
     ddf = dd.from_pandas(df, 3)
 
     result = ddf.quantile()
@@ -632,6 +647,7 @@ def test_dataframe_quantile():
     assert eq(ddf.quantile(axis=1), df.quantile(axis=1))
     assert raises(ValueError, lambda: ddf.quantile([0.25, 0.75], axis=1))
 
+
 def test_index():
     assert eq(d.index, full.index)
 
@@ -642,12 +658,12 @@ def test_loc():
 
     assert d.loc[5].divisions == (5, 5)
 
-    assert eq(d.loc[5], full.loc[5])
+    assert eq(d.loc[5], full.loc[5:5])
     assert eq(d.loc[3:8], full.loc[3:8])
     assert eq(d.loc[:8], full.loc[:8])
     assert eq(d.loc[3:], full.loc[3:])
 
-    assert eq(d.a.loc[5], full.a.loc[5])
+    assert eq(d.a.loc[5], full.a.loc[5:5])
     assert eq(d.a.loc[3:8], full.a.loc[3:8])
     assert eq(d.a.loc[:8], full.a.loc[:8])
     assert eq(d.a.loc[3:], full.a.loc[3:])
@@ -660,11 +676,26 @@ def test_loc():
     assert sorted(d.loc[5].dask) != sorted(d.loc[6].dask)
 
 
+def test_loc_non_informative_index():
+    df = pd.DataFrame({'x': [1, 2, 3, 4]}, index=[10, 20, 30, 40])
+    ddf = dd.from_pandas(df, npartitions=2, sort=True)
+    ddf.divisions = (None,) * 3
+    assert not ddf.known_divisions
+
+    ddf.loc[20:30].compute(get=dask.get)
+
+    assert eq(ddf.loc[20:30], df.loc[20:30])
+
+    df = pd.DataFrame({'x': [1, 2, 3, 4]}, index=[10, 20, 20, 40])
+    ddf = dd.from_pandas(df, npartitions=2, sort=True)
+    assert eq(ddf.loc[20], df.loc[20:20])
+
+
 def test_loc_with_text_dates():
     A = tm.makeTimeSeries(10).iloc[:5]
     B = tm.makeTimeSeries(10).iloc[5:]
     s = dd.Series({('df', 0): A, ('df', 1): B}, 'df', None,
-                  [A.index.min(), A.index.max(), B.index.max()])
+                  [A.index.min(), B.index.min(), B.index.max()])
 
     assert s.loc['2000': '2010'].divisions == s.divisions
     assert eq(s.loc['2000': '2010'], s)
@@ -689,7 +720,7 @@ def test_getitem():
                       columns=list('ABC'))
     ddf = dd.from_pandas(df, 2)
     assert eq(ddf['A'], df['A'])
-    tm.assert_series_equal(ddf['A']._pd, ddf._pd['A']) # check cache consistency
+    tm.assert_series_equal(ddf['A']._pd, ddf._pd['A'])# check cache consistency
 
     assert eq(ddf[['A', 'B']], df[['A', 'B']])
     tm.assert_frame_equal(ddf[['A', 'B']]._pd, ddf._pd[['A', 'B']])
@@ -755,6 +786,7 @@ def test_known_divisions():
                       ['a', 'b'], divisions=[0, 1])
     assert d.known_divisions
 
+
 def test_unknown_divisions():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]}),
            ('x', 1): pd.DataFrame({'a': [4, 5, 6], 'b': [3, 2, 1]}),
@@ -765,8 +797,6 @@ def test_unknown_divisions():
     assert eq(d.a.sum(), full.a.sum())
     assert eq(d.a + d.b + 1, full.a + full.b + 1)
 
-    assert raises(ValueError, lambda: d.loc[3])
-
 
 def test_concat2():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]}),
@@ -811,6 +841,7 @@ def test_concat2():
         with tm.assertRaisesRegexp(ValueError, msg):
             dd.concat(case, axis=1)
 
+
 def test_concat3():
     pdf1 = pd.DataFrame(np.random.randn(6, 5),
                         columns=list('ABCDE'), index=list('abcdef'))
@@ -840,6 +871,7 @@ def test_concat3():
     assert eq(dd.concat([ddf1, ddf2, ddf3], interleave_partitions=True),
               pd.concat([pdf1, pdf2, pdf3]))
 
+
 def test_concat4_interleave_partitions():
     pdf1 = pd.DataFrame(np.random.randn(10, 5),
                         columns=list('ABCDE'), index=list('abcdefghij'))
@@ -931,6 +963,7 @@ def test_concat5():
         assert eq(dd.concat(case, axis=1, join='inner'),
                   pd.concat(pdcase, axis=1, join='inner'))
 
+
 def test_append():
     df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6],
                        'b': [1, 2, 3, 4, 5, 6]})
@@ -944,6 +977,10 @@ def test_append():
     ddf = dd.from_pandas(df, 2)
     ddf2 = dd.from_pandas(df2, 2)
     ddf3 = dd.from_pandas(df3, 2)
+
+    s = pd.Series([7, 8], name=6, index=['a', 'b'])
+    assert eq(ddf.append(s), df.append(s))
+
     assert eq(ddf.append(ddf2), df.append(df2))
     assert eq(ddf.a.append(ddf2.a), df.a.append(df2.a))
     # different columns
@@ -957,11 +994,6 @@ def test_append():
     assert eq(ddf.append(df3), df.append(df3))
     assert eq(ddf.a.append(df3.b), df.a.append(df3.b))
 
-    s = pd.Series([7, 8], name=6, index=['a', 'b'])
-    assert eq(ddf.append(s), df.append(s))
-
-
-
     df4 = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6],
                         'b': [1, 2, 3, 4, 5, 6]},
                        index=[4, 5, 6, 7, 8, 9])
@@ -973,6 +1005,7 @@ def test_append():
     with tm.assertRaisesRegexp(ValueError, msg):
         ddf.append(ddf4)
 
+
 def test_append2():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]}),
            ('x', 1): pd.DataFrame({'a': [4, 5, 6], 'b': [3, 2, 1]}),
@@ -1053,7 +1086,6 @@ def test_set_partition_2():
 
 
 def test_repartition():
-
     def _check_split_data(orig, d):
         """Check data is split properly"""
         keys = [k for k in d.dask if k[0].startswith('repartition-split')]
@@ -1141,7 +1173,13 @@ def test_repartition():
 
 
 def test_repartition_divisions():
-    result = repartition_divisions([1, 3, 7], [1, 4, 6, 7], 'a', 'b', 'c')  # doctest: +SKIP
+    result = repartition_divisions([0, 6], [0, 6, 6], 'a', 'b', 'c')
+    assert result == {('b', 0): (_loc, ('a', 0), 0, 6, False),
+                      ('b', 1): (_loc, ('a', 0), 6, 6, True),
+                      ('c', 0): ('b', 0),
+                      ('c', 1): ('b', 1)}
+
+    result = repartition_divisions([1, 3, 7], [1, 4, 6, 7], 'a', 'b', 'c')
     assert result == {('b', 0): (_loc, ('a', 0), 1, 3, False),
                       ('b', 1): (_loc, ('a', 1), 3, 4, False),
                       ('b', 2): (_loc, ('a', 1), 4, 6, False),
@@ -1150,6 +1188,7 @@ def test_repartition_divisions():
                       ('c', 1): ('b', 2),
                       ('c', 2): ('b', 3)}
 
+
 def test_repartition_on_pandas_dataframe():
     df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')},
                       index=[10, 20, 30, 40, 50, 60])
@@ -1164,6 +1203,25 @@ def test_repartition_on_pandas_dataframe():
     assert eq(ddf, df.y)
 
 
+def test_repartition_npartitions():
+    df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')},
+                      index=[10, 20, 30, 40, 50, 60])
+    for n in [1, 2, 4, 5]:
+        for k in [1, 2, 4, 5]:
+            if k > n:
+                continue
+            a = dd.from_pandas(df, npartitions=n)
+            k = min(a.npartitions, k)
+
+            b = a.repartition(npartitions=k)
+            eq(a, b)
+            assert b.npartitions == k
+
+    a = dd.from_pandas(df, npartitions=1)
+    with pytest.raises(ValueError):
+        a.repartition(npartitions=5)
+
+
 def test_embarrassingly_parallel_operations():
     df = pd.DataFrame({'x': [1, 2, 3, 4, None, 6], 'y': list('abdabd')},
                       index=[10, 20, 30, 40, 50, 60])
@@ -1242,7 +1300,6 @@ def test_str_accessor():
 
 
 def test_empty_max():
-    df = pd.DataFrame({'x': [1, 2, 3]})
     a = dd.DataFrame({('x', 0): pd.DataFrame({'x': [1]}),
                       ('x', 1): pd.DataFrame({'x': []})}, 'x',
                       ['x'], [None, None, None])
@@ -1279,7 +1336,6 @@ def test_nlargest_series():
     assert eq(ss.nlargest(2), s.nlargest(2))
 
 
-
 def test_query():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [5, 6, 7, 8]})
     a = dd.from_pandas(df, npartitions=2)
@@ -1308,10 +1364,10 @@ def test_deterministic_reduction_names():
     assert a.x.max()._name == a.x.max()._name
     assert a.x.count()._name == a.x.count()._name
     # Test reduction without token string
-    assert sorted(reduction(a.x, len, np.sum).dask) !=\
-           sorted(reduction(a.x, np.sum, np.sum).dask)
-    assert sorted(reduction(a.x, len, np.sum).dask) ==\
-           sorted(reduction(a.x, len, np.sum).dask)
+    assert (sorted(reduction(a.x, len, np.sum).dask) !=
+            sorted(reduction(a.x, np.sum, np.sum).dask))
+    assert (sorted(reduction(a.x, len, np.sum).dask) ==
+            sorted(reduction(a.x, len, np.sum).dask))
 
 
 def test_deterministic_apply_concat_apply_names():
@@ -1320,17 +1376,17 @@ def test_deterministic_apply_concat_apply_names():
 
     assert sorted(a.x.nlargest(2).dask) == sorted(a.x.nlargest(2).dask)
     assert sorted(a.x.nlargest(2).dask) != sorted(a.x.nlargest(3).dask)
-    assert sorted(a.x.drop_duplicates().dask) == \
-           sorted(a.x.drop_duplicates().dask)
-    assert sorted(a.groupby('x').y.mean().dask) == \
-           sorted(a.groupby('x').y.mean().dask)
+    assert (sorted(a.x.drop_duplicates().dask) ==
+            sorted(a.x.drop_duplicates().dask))
+    assert (sorted(a.groupby('x').y.mean().dask) ==
+            sorted(a.groupby('x').y.mean().dask))
     # Test aca without passing in token string
     f = lambda a: a.nlargest(5)
     f2 = lambda a: a.nlargest(3)
-    assert sorted(aca(a.x, f, f, a.x.name).dask) !=\
-           sorted(aca(a.x, f2, f2, a.x.name).dask)
-    assert sorted(aca(a.x, f, f, a.x.name).dask) ==\
-           sorted(aca(a.x, f, f, a.x.name).dask)
+    assert (sorted(aca(a.x, f, f, a.x.name).dask) !=
+            sorted(aca(a.x, f2, f2, a.x.name).dask))
+    assert (sorted(aca(a.x, f, f, a.x.name).dask) ==
+            sorted(aca(a.x, f, f, a.x.name).dask))
 
 
 def test_gh_517():
@@ -1382,6 +1438,7 @@ def test_to_frame():
     assert eq(s.to_frame(), a.to_frame())
     assert eq(s.to_frame('bar'), a.to_frame('bar'))
 
+
 def test_apply():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
@@ -1511,7 +1568,7 @@ def test_index_time_properties():
 
 
 @pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.16.2',
-        reason="nlargest not in pandas pre 0.16.2")
+                    reason="nlargest not in pandas pre 0.16.2")
 def test_nlargest():
     from string import ascii_lowercase
     df = pd.DataFrame({'a': np.random.permutation(10),
@@ -1524,7 +1581,7 @@ def test_nlargest():
 
 
 @pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.16.2',
-        reason="nlargest not in pandas pre 0.16.2")
+                    reason="nlargest not in pandas pre 0.16.2")
 def test_nlargest_multiple_columns():
     from string import ascii_lowercase
     df = pd.DataFrame({'a': np.random.permutation(10),
@@ -1553,12 +1610,14 @@ def test_dataframe_compute_forward_kwargs():
     x = dd.from_pandas(pd.DataFrame({'a': range(10)}), npartitions=2).a.sum()
     x.compute(bogus_keyword=10)
 
+
 def test_series_iteritems():
     df = pd.DataFrame({'x': [1, 2, 3, 4]})
     ddf = dd.from_pandas(df, npartitions=2)
     for (a, b) in zip(df['x'].iteritems(), ddf['x'].iteritems()):
         assert a == b
 
+
 def test_dataframe_iterrows():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
@@ -1566,6 +1625,7 @@ def test_dataframe_iterrows():
     for (a, b) in zip(df.iterrows(), ddf.iterrows()):
         tm.assert_series_equal(a[1], b[1])
 
+
 def test_dataframe_itertuples():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
@@ -1574,30 +1634,30 @@ def test_dataframe_itertuples():
         assert a == b
 
 
-def test_from_imperative():
+def test_from_delayed():
     from dask import do
     dfs = [do(tm.makeTimeDataFrame)(i) for i in range(1, 5)]
-    df = dd.from_imperative(dfs, columns=['A', 'B', 'C', 'D'])
+    df = dd.from_delayed(dfs, columns=['A', 'B', 'C', 'D'])
 
     assert (df.compute().columns == df.columns).all()
     assert list(df.map_partitions(len).compute()) == [1, 2, 3, 4]
 
     ss = [df.A for df in dfs]
-    s = dd.from_imperative(ss, columns='A')
+    s = dd.from_delayed(ss, columns='A')
 
     assert s.compute().name == s.name
     assert list(s.map_partitions(len).compute()) == [1, 2, 3, 4]
 
-    df = dd.from_imperative(dfs, tm.makeTimeDataFrame(1))
+    df = dd.from_delayed(dfs, tm.makeTimeDataFrame(1))
     assert df._known_dtype
     assert list(df.columns) == ['A', 'B', 'C', 'D']
 
 
-def test_to_imperative():
-    from dask.imperative import Value
+def test_to_delayed():
+    from dask.delayed import Value
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
-    a, b = ddf.to_imperative()
+    a, b = ddf.to_delayed()
     assert isinstance(a, Value)
     assert isinstance(b, Value)
 
@@ -1611,3 +1671,47 @@ def test_astype():
 
     assert eq(a.astype(float), df.astype(float))
     assert eq(a.x.astype(float), df.x.astype(float))
+
+
+def test_groupby_callable():
+    a = pd.DataFrame({'x': [1, 2, 3, None], 'y': [10, 20, 30, 40]},
+                      index=[1, 2, 3, 4])
+    b = dd.from_pandas(a, 2)
+
+    def iseven(x):
+        return x % 2 == 0
+
+    assert eq(a.groupby(iseven).y.sum(),
+              b.groupby(iseven).y.sum())
+    assert eq(a.y.groupby(iseven).sum(),
+              b.y.groupby(iseven).sum())
+
+
+def test_set_index_sorted_true():
+    df = pd.DataFrame({'x': [1, 2, 3, 4],
+                       'y': [10, 20, 30, 40],
+                       'z': [4, 3, 2, 1]})
+    a = dd.from_pandas(df, 2, sort=False)
+    assert not a.known_divisions
+
+    b = a.set_index('x', sorted=True)
+    assert b.known_divisions
+    assert set(a.dask).issubset(set(b.dask))
+
+    for drop in [True, False]:
+        eq(a.set_index('x', drop=drop),
+           df.set_index('x', drop=drop))
+        eq(a.set_index(a.x, sorted=True, drop=drop),
+           df.set_index(df.x, drop=drop))
+        eq(a.set_index(a.x + 1, sorted=True, drop=drop),
+           df.set_index(df.x + 1, drop=drop))
+
+    with pytest.raises(ValueError):
+        a.set_index(a.z, sorted=True)
+
+
+def test_methods_tokenize_differently():
+    df = pd.DataFrame({'x': [1, 2, 3, 4]})
+    df = dd.from_pandas(df, npartitions=1)
+    assert (df.x.map_partitions(pd.Series.min)._name !=
+            df.x.map_partitions(pd.Series.max)._name)
